﻿using Microsoft.Extensions.Logging;
using PizzeriaContracts.BusinessLogicsContracts;
using PizzeriaContracts.SearchModels;
using PizzeriaContracts.ViewModels;
using PizzeriaDataModels;


namespace Pizzeria
{
    public partial class FormShopPizza : Form
    {
        ILogger<FormShopPizza> _logger;
        IShopLogic _shopLogic;
        IPizzaLogic _pizzaLogic;
        List<PizzaViewModel> _listPizza;

        public int ShopId
        {
            get
            {
                return Convert.ToInt32(comboBoxShop.SelectedValue);
            }
            set
            {
                comboBoxShop.SelectedValue = value;
            }
        }

        public FormShopPizza(ILogger<FormShopPizza> logger, IShopLogic shopLigic, IPizzaLogic pizzaLigic)
        {
            InitializeComponent();

            _logger = logger;
            _shopLogic = shopLigic;
            _pizzaLogic = pizzaLigic;

            List<ShopViewModel> _listShop = shopLigic.ReadList(null);
            List<PizzaViewModel> _listPizza = pizzaLigic.ReadList(null);
            if (_listShop != null)
            {
                comboBoxShop.DisplayMember = "ShopName";
                comboBoxShop.ValueMember = "Id";
                comboBoxShop.DataSource = _listShop;
                comboBoxShop.SelectedItem = null;
            }
            if (_listPizza != null)
            {
                comboBoxPizza.DisplayMember = "PizzaName";
                comboBoxPizza.ValueMember = "Id";
                comboBoxPizza.DataSource = _listPizza;
                comboBoxPizza.SelectedItem = null;
            }
        }

        private void buttonSave_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(textBoxCount.Text))
            {
                MessageBox.Show("Заполните поле Количество", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (comboBoxShop.SelectedValue == null)
            {
                MessageBox.Show("Выберите магазин", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (comboBoxPizza.SelectedValue == null)
            {
                MessageBox.Show("Выберите пиццу", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }


            try
            {
                int count = Convert.ToInt32(textBoxCount.Text);

                bool res = _shopLogic.AddPizza(
                    new() { Id = Convert.ToInt32(comboBoxShop.SelectedValue) },
                    _pizzaLogic.ReadElement(new() { Id = Convert.ToInt32(comboBoxPizza.SelectedValue) }),
                    count
                );

                if (!res)
                {
                    throw new Exception("Ошибка при пополнении. Дополнительная информация в логах");
                }

                MessageBox.Show("Пополнение прошло успешно");
                DialogResult = DialogResult.OK;
                Close();

            }
            catch (Exception err)
            {
                MessageBox.Show("Ошибка пополнения");
                _logger.LogError(err, "Ошибка пополнения");
                return;
            }
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }
    }
}