﻿using PizzeriaContracts.BindingModels;
using PizzeriaContracts.SearchModels;
using PizzeriaContracts.StoragesContracts;
using PizzeriaContracts.ViewModels;
using PizzeriaDatabaseImplement.models;
using Microsoft.EntityFrameworkCore;
using PizzeriaDataModels;

namespace PizzeriaDatabaseImplement.Implements
{
    public class ShopStorage : IShopStorage
    {
        public List<ShopViewModel> GetFullList()
        {
            using var context = new PizzeriaDatabase();
            return context.Shops
             .Include(x => x.Pizzas)
            .ThenInclude(x => x.Pizza)
            .ToList()
            .Select(x => x.GetViewModel)
            .ToList();
        }
        public List<ShopViewModel> GetFilteredList(ShopSearchModel model)
        {
            if (string.IsNullOrEmpty(model.ShopName))
            {
                return new();
            }
            using var context = new PizzeriaDatabase();
            return context.Shops
             .Include(x => x.Pizzas)
            .ThenInclude(x => x.Pizza)
            .Where(x => x.ShopName.Contains(model.ShopName))
            .ToList()
            .Select(x => x.GetViewModel)
            .ToList();
        }
        public ShopViewModel? GetElement(ShopSearchModel model)
        {
            if (string.IsNullOrEmpty(model.ShopName) &&
            !model.Id.HasValue)
            {
                return null;
            }
            using var context = new PizzeriaDatabase();
            return context.Shops
             .Include(x => x.Pizzas)
             .ThenInclude(x => x.Pizza)
             .FirstOrDefault(x => (!string.IsNullOrEmpty(model.ShopName) &&
                                    x.ShopName == model.ShopName) ||
                                    (model.Id.HasValue && x.Id == model.Id))?.GetViewModel;

        }
        public ShopViewModel? Insert(ShopBindingModel model)
        {
            using var context = new PizzeriaDatabase();
            var newShop = Shop.Create(context, model);

            if (context.Shops.Any(x => x.ShopName == newShop.ShopName))
            {
                throw new Exception("Магазин с таким названием уже существует");
            }

            if (newShop == null)
            {
                return null;
            }
            context.Shops.Add(newShop);
            context.SaveChanges();
            return newShop.GetViewModel;
        }
        public ShopViewModel? Update(ShopBindingModel model)
        {
            using var context = new PizzeriaDatabase();
            using var transaction = context.Database.BeginTransaction();
            try
            {
                var shop = context.Shops.FirstOrDefault(rec =>
                rec.Id == model.Id);
                if (shop == null)
                {
                    return null;
                }
                shop.Update(model);
                context.SaveChanges();
                shop.UpdatePizzas(context, model);
                transaction.Commit();
                return shop.GetViewModel;
            }
            catch
            {
                transaction.Rollback();
                throw;
            }
        }
        public ShopViewModel? Delete(ShopBindingModel model)
        {
            using var context = new PizzeriaDatabase();
            var element = context.Shops
            .Include(x => x.Pizzas)
            .FirstOrDefault(rec => rec.Id == model.Id);
            if (element != null)
            {
                context.Shops.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }
            return null;
        }

        public bool TrySellPizza(IPizzaModel pizza, int count)
        {
            using var context = new PizzeriaDatabase();
            using var transaction = context.Database.BeginTransaction();

            foreach (var shopPizzas in context.ShopPizzas.Where(x => x.PizzaId == pizza.Id))
            {
                var min = Math.Min(count, shopPizzas.Count);
                shopPizzas.Count -= min;
                count -= min;

                if (count <= 0)
                {
                    break;
                }
            }

            if (count == 0)
            {
                context.SaveChanges();
                transaction.Commit();
            }

            else
            {
                transaction.Rollback();
            }

            if (count > 0)
            {
                return false;
            }

            return true;
        }

        public bool TryRestoreShops(IPizzaModel pizza, int count)
        {
            throw new NotImplementedException();
        }
    }
}
