﻿using PizzeriaContracts.BindingModels;
using PizzeriaContracts.ViewModels;
using PizzeriaDataModels;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;

namespace PizzeriaDatabaseImplement.models
{
    public class Shop : IShopModel
    {
        public int Id { get; set; }
        [Required]
        public string ShopName { get; set; } = string.Empty;
        [Required]
        public string Addres { get; set; } = string.Empty;
        [Required]
        public DateTime OpenTime { get; set; }
        [Required]
        public int Capacity { get; set; }
        private Dictionary<int, (IPizzaModel, int)>? _shopPizzas = null;
        [NotMapped]
        public Dictionary<int, (IPizzaModel, int)> ShopPizzas
        {
            get
            {
                if (_shopPizzas == null)
                {
                    _shopPizzas = Pizzas.ToDictionary(x => x.PizzaId, x => (x.Pizza as IPizzaModel, x.Count));
                }
                return _shopPizzas;
            }
        }
        [ForeignKey("ShopId")]
        public virtual List<ShopPizza> Pizzas { get; set; } = new();
        public static Shop Create(PizzeriaDatabase context, ShopBindingModel model)
        {
            return new Shop()
            {
                Id = model.Id,
                ShopName = model.ShopName,
                Addres = model.Addres,
                OpenTime = model.OpenTime,
                Capacity = model.Capacity,
                Pizzas = model.ShopPizzas.Select(x => new ShopPizza
                {
                    Pizza = context.Pizzas.First(y => y.Id == x.Key),
                    Count = x.Value.Item2
                }).ToList()
            };
        }
        public void Update(ShopBindingModel model)
        {
            if (model == null)
            {
                return;
            }
            ShopName = model.ShopName;
            Addres = model.Addres;
            OpenTime = model.OpenTime;
            Capacity = model.Capacity;
        }
        public ShopViewModel GetViewModel => new()
        {
            Id = Id,
            ShopName = ShopName,
            Addres = Addres,
            OpenTime = OpenTime,
            Capacity = Capacity,
            ShopPizzas = ShopPizzas
        };
        public void UpdatePizzas(PizzeriaDatabase context, ShopBindingModel model)
        {
            var shopPizzas = context.ShopPizzas.Where(rec => rec.ShopId == model.Id).ToList();
            if (shopPizzas != null && shopPizzas.Count > 0)
            { // удалили те, которых нет в модели
                context.ShopPizzas.RemoveRange(shopPizzas.Where(rec
               => !model.ShopPizzas.ContainsKey(rec.PizzaId)));
                context.SaveChanges();
                // обновили количество у существующих записей
                foreach (var updatePizza in shopPizzas)
                {
                    updatePizza.Count =
                   model.ShopPizzas[updatePizza.PizzaId].Item2;
                    model.ShopPizzas.Remove(updatePizza.PizzaId);
                }
                context.SaveChanges();
            }
            var shop = context.Shops.First(x => x.Id == Id);
            foreach (var pc in model.ShopPizzas)
            {
                context.ShopPizzas.Add(new ShopPizza
                {
                    Shop = shop,
                    Pizza = context.Pizzas.First(x => x.Id == pc.Key),
                    Count = pc.Value.Item2
                });
                context.SaveChanges();
            }
            _shopPizzas = null;
        }
    }
}
